@@ -1,14 +1,14 @@
 # -*- coding: latin-1 -*-
 # Copyright (c) 2008-2009 Michael Howitz
 # See also LICENSE.txt
-# $Id: tests.py 913 2009-01-02 12:47:38Z mac $
+# $Id: tests.py 935 2009-01-08 10:19:09Z mac $
 
 import unittest
 import zope.interface.verify
 
 import icemac.addressbook.export.interfaces
 import icemac.addressbook.export.xls.simple
-import icemac.addressbook.tests
+import icemac.addressbook.testing
 
 
 class TestInterfaces(unittest.TestCase):
@@ -28,5 +28,5 @@ class TestInterfaces(unittest.TestCase):
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(TestInterfaces))
-    suite.layer = icemac.addressbook.tests.AddressBookUnitTests
+    suite.layer = icemac.addressbook.testing.AddressBookUnitTests
     return suite
