@@ -13,7 +13,7 @@
 ##############################################################################
 """Widget Framework Implementation
 
-$Id: widget.py 93844 2008-12-10 15:54:24Z adamg $
+$Id: widget.py 96365 2009-02-10 03:48:08Z nadako $
 """
 __docformat__ = "reStructuredText"
 
@@ -22,7 +22,7 @@ import zope.component
 import zope.location
 import zope.schema.interfaces
 from zope.pagetemplate.interfaces import IPageTemplate
-from z3c.pt import compat as viewpagetemplatefile
+from z3c import ptcompat as viewpagetemplatefile
 from zope.i18n import translate
 from zope.schema.fieldproperty import FieldProperty
 
@@ -52,6 +52,7 @@ class Widget(zope.location.Location):
     value = FieldProperty(interfaces.IWidget['value'])
     template = None
     ignoreRequest = FieldProperty(interfaces.IWidget['ignoreRequest'])
+    setErrors = FieldProperty(interfaces.IWidget['setErrors'])
 
     # The following attributes are for convenience. They are declared in
     # extensions to the simple widget.
@@ -73,14 +74,15 @@ class Widget(zope.location.Location):
     def update(self):
         """See z3c.form.interfaces.IWidget."""
         # Step 1: Determine the value.
-        value = interfaces.NOVALUE
+        value = interfaces.NO_VALUE
         lookForDefault = False
         # Step 1.1: If possible, get a value from the request
         if not self.ignoreRequest:
             #at this turn we do not need errors to be set on widgets
             #errors will be set when extract gets called from form.extractData
-            widget_value = self.extract(setErrors=False)
-            if widget_value is not interfaces.NOVALUE:
+            self.setErrors = False
+            widget_value = self.extract()
+            if widget_value is not interfaces.NO_VALUE:
                 # Once we found the value in the request, it takes precendence
                 # over everything and nothing else has to be done.
                 self.value = widget_value
@@ -88,7 +90,7 @@ class Widget(zope.location.Location):
         # Step 1.2: If we have a widget with a field and we have no value yet,
         #           we have some more possible locations to get the value
         if (interfaces.IFieldWidget.providedBy(self) and
-            value is interfaces.NOVALUE and
+            value is interfaces.NO_VALUE and
             value is not PLACEHOLDER):
             # Step 1.2.1: If the widget knows about its context and the
             #              context is to be used to extract a value, get
@@ -103,20 +105,20 @@ class Widget(zope.location.Location):
             # NOTE: It should check field.default is not missing_value, but
             # that requires fixing zope.schema first
             if ((value is self.field.missing_value or
-                 value is interfaces.NOVALUE) and
+                 value is interfaces.NO_VALUE) and
                 self.field.default is not None):
                 value = self.field.default
                 lookForDefault = True
         # Step 1.3: If we still have not found a value, then we try to get it
         #           from an attribute value
-        if value is interfaces.NOVALUE or lookForDefault:
+        if value is interfaces.NO_VALUE or lookForDefault:
             adapter = zope.component.queryMultiAdapter(
                 (self.context, self.request, self.form, self.field, self),
                 interfaces.IValue, name='default')
             if adapter:
                 value = adapter.get()
         # Step 1.4: Convert the value to one that the widget can understand
-        if value not in (interfaces.NOVALUE, PLACEHOLDER):
+        if value not in (interfaces.NO_VALUE, PLACEHOLDER):
             converter = interfaces.IDataConverter(self)
             self.value = converter.toWidgetValue(value)
         # Step 2: Update selected attributes
@@ -138,7 +140,7 @@ class Widget(zope.location.Location):
                 IPageTemplate, name=self.mode)
         return template(self)
 
-    def extract(self, default=interfaces.NOVALUE, setErrors=True):
+    def extract(self, default=interfaces.NO_VALUE):
         """See z3c.form.interfaces.IWidget."""
         return self.request.get(self.name, default)
 
@@ -197,7 +199,7 @@ class SequenceWidget(Widget):
         self.updateTerms()
         super(SequenceWidget, self).update()
 
-    def extract(self, default=interfaces.NOVALUE, setErrors=True):
+    def extract(self, default=interfaces.NO_VALUE):
         """See z3c.form.interfaces.IWidget."""
         if (self.name not in self.request and
             self.name+'-empty-marker' in self.request):
@@ -242,15 +244,15 @@ class MultiWidget(Widget):
     zope.interface.implements(interfaces.IMultiWidget)
 
     allowAdding = True
-    # you set showLabel to False or use another template for disable (sub)
-    # widget labels
-    showLabel = True
+    allowRemoving = True
+
     widgets = None
-    _value = []
+    _value = None
 
     def __init__(self, request):
         super(MultiWidget, self).__init__(request)
         self.widgets = []
+        self._value = []
 
     @property
     def counterName(self):
@@ -265,13 +267,10 @@ class MultiWidget(Widget):
 
     def getWidget(self, idx):
         """Setup widget based on index id with or without value."""
-        id = '%s-%i' % (self.id, idx)
-        name = '%s.%i' % (self.name, idx)
         valueType = self.field.value_type
         widget = zope.component.getMultiAdapter((valueType, self.request),
             interfaces.IFieldWidget)
-        widget.name = name
-        widget.id = id
+        self.setName(widget, idx)
         widget.mode = self.mode
         #set widget.form (objectwidget needs this)
         if interfaces.IFormAware.providedBy(self):
@@ -281,6 +280,10 @@ class MultiWidget(Widget):
         widget.update()
         return widget
 
+    def setName(self, widget, idx):
+        widget.name = '%s.%i' % (self.name, idx)
+        widget.id = '%s-%i' % (self.id, idx)
+
     def appendAddingWidget(self):
         """Simply append a new empty widget with correct (counter) name."""
         # since we start with idx 0 (zero) we can use the len as next idx
@@ -288,7 +291,7 @@ class MultiWidget(Widget):
         widget = self.getWidget(idx)
         self.widgets.append(widget)
 
-    def applyValue(self, widget, value=interfaces.NOVALUE):
+    def applyValue(self, widget, value=interfaces.NO_VALUE):
         """Validate and apply value to given widget.
 
         This method gets called on any multi widget value change and is
@@ -298,11 +301,11 @@ class MultiWidget(Widget):
         nothing outside this multi widget does know something about our
         internal sub widgets.
         """
-        if value is not interfaces.NOVALUE:
+        if value is not interfaces.NO_VALUE:
             try:
                 # convert widget value to field value
                 converter = interfaces.IDataConverter(widget)
-                value = converter.toFieldValue(value)
+                fvalue = converter.toFieldValue(value)
                 # validate field value
                 zope.component.getMultiAdapter(
                     (self.context,
@@ -310,9 +313,9 @@ class MultiWidget(Widget):
                      self.form,
                      getattr(widget, 'field', None),
                      widget),
-                    interfaces.IValidator).validate(value)
+                    interfaces.IValidator).validate(fvalue)
                 # convert field value to widget value
-                widget.value = converter.toWidgetValue(value)
+                widget.value = converter.toWidgetValue(fvalue)
             except (zope.schema.ValidationError, ValueError), error:
                 # on exception, setup the widget error message
                 view = zope.component.getMultiAdapter(
@@ -328,18 +331,32 @@ class MultiWidget(Widget):
         """
         oldLen = len(self.widgets)
         self.widgets = []
+        idx = 0
         if self.value:
-            for idx, v in enumerate(self.value):
+            for v in self.value:
                 widget = self.getWidget(idx)
                 self.applyValue(widget, v)
                 self.widgets.append(widget)
+                idx += 1
         missing = oldLen - len(self.widgets)
         if missing > 0:
             # add previous existing new added widgtes
-            for i in range(missing):
-                idx += 1
+            for i in xrange(missing):
                 widget = self.getWidget(idx)
                 self.widgets.append(widget)
+                idx += 1
+
+    def updateAllowAddRemove(self):
+        """Update the allowAdding/allowRemoving attributes
+        basing on field constraints and current number of widgets
+        """
+        if not zope.schema.interfaces.IMinMaxLen.providedBy(self.field):
+            return
+        max_length = self.field.max_length
+        min_length = self.field.min_length
+        num_items = len(self.widgets)
+        self.allowAdding = bool((not max_length) or (num_items < max_length))
+        self.allowRemoving = bool(num_items and (num_items > min_length))
 
     @apply
     def value():
@@ -352,7 +369,7 @@ class MultiWidget(Widget):
             self.updateWidgets()
         return property(get, set)
 
-    def extract(self, default=interfaces.NOVALUE, setErrors=True):
+    def extract(self, default=interfaces.NO_VALUE):
         # This method is responsible to get the widgets value based on the
         # request and nothing else.
         # We have to setup the widgets for extract their values, because we
@@ -362,7 +379,7 @@ class MultiWidget(Widget):
         # which whould generate a different set of widgets.
         if self.request.get(self.counterName) is None:
             # counter marker not found
-            return interfaces.NOVALUE
+            return interfaces.NO_VALUE
         counter = int(self.request.get(self.counterName, 0))
         values = []
         append = values.append
@@ -372,7 +389,7 @@ class MultiWidget(Widget):
             append(widget.value)
         if len(values) == 0:
             # no multi value found
-            return interfaces.NOVALUE
+            return interfaces.NO_VALUE
         return values
 
 
